:- module(
  c_compiler,
  [
    % Public API
    compile_c_program/3,
    compile_cpp_program/3,
    gsl_compile_options/1,
    ppl_compile_options/1
  ]
).


% Public API


compile_c_program(SourceFiles, Options, ExecutableFilename) :-
  compile_program('gcc', SourceFiles, Options, ExecutableFilename).


compile_cpp_program(SourceFiles, Options, ExecutableFilename) :-
  compile_program(
    'g++', SourceFiles, ['--std=c++11', '-Dtypeof(x)=decltype(x)' | Options], ExecutableFilename
  ).


gsl_compile_options(['-lgsl', '-lgslcblas', '-lm']).


ppl_compile_options(['-lppl_c', '-lppl', '-lgmpxx', '-lgmp']).


% Private predicates


compile_program(Compiler, SourceFiles, Options, ExecutableFilename) :-
  library_path(LibraryPath),
  findall(
    AbsoluteSourceFile,
    (
      member(SourceFile, SourceFiles),
      absolute_file_name(
          SourceFile, AbsoluteSourceFile, [relative_to(LibraryPath)]
      )
    ),
    AbsoluteSourceFiles
  ),
  append(AbsoluteSourceFiles, Options, AllOptions),
  (
    debugging('c_compiler')
  ->
    AllOptions0 = ['-g' | AllOptions]
  ;
    AllOptions0 = AllOptions
  ),
  call_subprocess(
    path(Compiler), ['-o', ExecutableFilename, '-I.' | AllOptions0], []
  ).
