:- module(
  odefunction,
  [
    add_function/1
  ]
).


:- devdoc('\\section{Commands}').


add_function(FunctionList) :-
  biocham_command(*),
  type(FunctionList, '*'(term = term)),
  doc('
    adds reactions to compute the result of a function of the current variables in the concentration of a new variable, at steady state.
    \\begin{example}
  '),
  biocham_silent(clear_model),
  biocham(present(x,1)),
  biocham(present(y,3)),
  biocham(add_function(z=x+y)),
  biocham(list_ode),
  biocham(list_model),
  biocham(numerical_simulation(time: 10)),
  biocham(plot),
  doc('
    \\end{example}
  '),
  new_ode_system(OdeSystem),
  create_ode(FunctionList, OdeSystem),
  import_reactions_from_ode_system(OdeSystem),
  delete_item(OdeSystem).


create_ode(_, OdeSystem) :-
  enumerate_molecules(L), % useless if item backtracks on Object
  member(Object, L),
  get_initial_concentration(Object, Concentration),
  set_ode_initial_value(OdeSystem, Object, Concentration),
  fail.

create_ode(FunctionList, OdeSystem) :-
  member(Variable = _, FunctionList),
  set_ode_initial_value(OdeSystem, Variable, 0),
  fail.

create_ode(FunctionList, OdeSystem) :-
  member(Variable = Value, FunctionList),
  add_ode(OdeSystem, d(Variable)/dt = Value - Variable),
  fail.

create_ode(_,_).
